@@ -139,11 +139,13 @@ class KeychainCoordinator:NSObject, PAPasscodeViewControllerDelegate {
     }
 
     private func passcodeFromKeychain() -> String {
-        if let item = try? XKKeychainGenericPasswordItem(forService: KeychainCoordinator.passcodeService, account: KeychainCoordinator.passcodeService) {
-            return item.secret.stringValue
-        }
-        assert(false, "Couldn't retrieve item from Keychain! If passcodeLockEnabled we should have an item and secret")
+      do {
+        let item = try XKKeychainGenericPasswordItem(forService: KeychainCoordinator.passcodeService, account: KeychainCoordinator.passcodeService)
+        return item.secret.stringValue
+      } catch let error {
+        assert(false, "Couldn't retrieve item from Keychain! If passcodeLockEnabled we should have an item and secret. Error was \(error)")
         return ""
+      }
     }
 
     //MARK: PAPassCodeDelegate
