@@ -10,6 +10,8 @@
  *****************************************************************************/
 
 #import "VLCNetworkLoginViewFieldCell.h"
+#import "VLC_iOS-Swift.h"
+
 NSString * const kVLCNetworkLoginViewFieldCellIdentifier = @"VLCNetworkLoginViewFieldCellIdentifier";
 
 @interface VLCNetworkLoginViewFieldCell () <UITextFieldDelegate>
@@ -39,22 +41,20 @@ NSString * const kVLCNetworkLoginViewFieldCellIdentifier = @"VLCNetworkLoginView
     self.textField.textColor = [UIColor whiteColor];
     [self addSubview:_textField];
     
-    NSObject *guide = self;
+    id<VLCLayoutAnchorContainer> guide = self;
     if (@available(iOS 11.0, *)) {
         guide = self.safeAreaLayoutGuide;
     }
-
-    [self addConstraints:@[
-                           [NSLayoutConstraint constraintWithItem:darkView attribute:NSLayoutAttributeLeft relatedBy:NSLayoutRelationEqual toItem:self attribute:NSLayoutAttributeLeft multiplier:1 constant:0],
-                           [NSLayoutConstraint constraintWithItem:darkView attribute:NSLayoutAttributeTop relatedBy:NSLayoutRelationEqual toItem:self attribute:NSLayoutAttributeTop multiplier:1 constant:0],
-                           [NSLayoutConstraint constraintWithItem:darkView attribute:NSLayoutAttributeRight relatedBy:NSLayoutRelationEqual toItem:self attribute:NSLayoutAttributeRight multiplier:1 constant:0],
-                           [NSLayoutConstraint constraintWithItem:darkView attribute:NSLayoutAttributeBottom relatedBy:NSLayoutRelationEqual toItem:self attribute:NSLayoutAttributeBottom multiplier:1 constant:-1],
-
-                           [NSLayoutConstraint constraintWithItem:self.textField attribute:NSLayoutAttributeLeft relatedBy:NSLayoutRelationEqual toItem:guide attribute:NSLayoutAttributeLeft multiplier:1 constant:8.0],
-                           [NSLayoutConstraint constraintWithItem:self.textField attribute:NSLayoutAttributeTop relatedBy:NSLayoutRelationEqual toItem:guide attribute:NSLayoutAttributeTop multiplier:1 constant:0],
-                           [NSLayoutConstraint constraintWithItem:self.textField attribute:NSLayoutAttributeRight relatedBy:NSLayoutRelationEqual toItem:guide attribute:NSLayoutAttributeRight multiplier:1 constant:8.0],
-                           [NSLayoutConstraint constraintWithItem:self.textField attribute:NSLayoutAttributeBottom relatedBy:NSLayoutRelationEqual toItem:guide attribute:NSLayoutAttributeBottom multiplier:1 constant:0]
-                           ]];
+    [NSLayoutConstraint activateConstraints:@[
+                                              [darkView.leftAnchor constraintEqualToAnchor:self.leftAnchor],
+                                              [darkView.topAnchor constraintEqualToAnchor:self.topAnchor],
+                                              [darkView.rightAnchor constraintEqualToAnchor:self.rightAnchor],
+                                              [darkView.bottomAnchor constraintEqualToAnchor:self.bottomAnchor constant:-1],
+                                              [self.textField.leftAnchor constraintEqualToAnchor:guide.leftAnchor constant:8.0],
+                                              [self.textField.topAnchor constraintEqualToAnchor:guide.topAnchor],
+                                              [self.textField.rightAnchor constraintEqualToAnchor:guide.rightAnchor constant:8.0],
+                                              [self.textField.bottomAnchor constraintEqualToAnchor:guide.bottomAnchor],
+                                              ]];
 }
 
 - (void)prepareForReuse
