@@ -110,22 +110,12 @@ NSString *const VLCDropboxSessionWasAuthorized = @"VLCDropboxSessionWasAuthorize
     // enable crash preventer
     void (^setupBlock)() = ^{
         __weak typeof(self) weakSelf = self;
-        void (^setupLibraryBlock)() = ^{
-            _libraryViewController = [[VLCLibraryViewController alloc] init];
-            UINavigationController *navCon = [[UINavigationController alloc] initWithRootViewController:_libraryViewController];
-
-            VLCSidebarController *sidebarVC = [VLCSidebarController sharedInstance];
-            sidebarVC.contentViewController = navCon;
-
-            VLCPlayerDisplayController *playerDisplayController = [VLCPlayerDisplayController sharedInstance];
-            playerDisplayController.childViewController = sidebarVC.fullViewController;
-
-            weakSelf.window.rootViewController = playerDisplayController;
-        };
         UINavigationController *navCon = [[UINavigationController alloc] initWithRootViewController:[[UIViewController alloc] init]];
         self.window.rootViewController = navCon;
         [self.window makeKeyAndVisible];
-        [self validatePasscodeIfNeededWithCompletion:setupLibraryBlock];
+        [self validatePasscodeIfNeededWithCompletion:^{
+            [weakSelf setupLibrary];
+        }];
 
         BOOL spotlightEnabled = ![VLCKeychainCoordinator passcodeLockEnabled];
         [[MLMediaLibrary sharedMediaLibrary] setSpotlightIndexingEnabled:spotlightEnabled];
@@ -202,6 +192,20 @@ NSString *const VLCDropboxSessionWasAuthorized = @"VLCDropboxSessionWasAuthorize
     return YES;
 }
 
+- (void)setupLibrary
+{
+    _libraryViewController = [[VLCLibraryViewController alloc] init];
+    UINavigationController *navCon = [[UINavigationController alloc] initWithRootViewController:_libraryViewController];
+
+    VLCSidebarController *sidebarVC = [VLCSidebarController sharedInstance];
+    sidebarVC.contentViewController = navCon;
+
+    VLCPlayerDisplayController *playerDisplayController = [VLCPlayerDisplayController sharedInstance];
+    playerDisplayController.childViewController = sidebarVC.fullViewController;
+
+    _window.rootViewController = playerDisplayController;
+}
+
 - (void)setupAppearence
 {
     UIColor *vlcOrange = [UIColor VLCOrangeTintColor];
@@ -215,6 +219,9 @@ NSString *const VLCDropboxSessionWasAuthorized = @"VLCDropboxSessionWasAuthorize
     [[UINavigationBar appearance] setBarTintColor:vlcOrange];
     [[UINavigationBar appearanceWhenContainedInInstancesOfClasses:@[[VLCPlaybackNavigationController class]]] setBarTintColor: nil];
     [[UINavigationBar appearance] setTintColor:[UIColor whiteColor]];
+    if (@available(iOS 11.0, *)) {
+        [[UINavigationBar appearanceWhenContainedInInstancesOfClasses:@[[UIDocumentBrowserViewController class]]] setTintColor: vlcOrange];
+    }
     [[UINavigationBar appearance] setTitleTextAttributes: @{ NSForegroundColorAttributeName : [UIColor whiteColor] }];
     // For the edit selection indicators
     [[UITableView appearance] setTintColor:vlcOrange];
@@ -277,7 +284,9 @@ continueUserActivity:(NSUserActivity *)userActivity
             APLog(@"%s file not found: %@",__PRETTY_FUNCTION__,userActivity);
             return NO;
         }
-        [[VLCPlaybackController sharedInstance] openMediaLibraryObject:managedObject];
+        [self validatePasscodeIfNeededWithCompletion:^{
+            [[VLCPlaybackController sharedInstance] openMediaLibraryObject:managedObject];
+        }];
         return YES;
     }
     return NO;
@@ -294,6 +303,17 @@ didFailToContinueUserActivityWithType:(NSString *)userActivityType
 
 - (BOOL)application:(UIApplication *)app openURL:(NSURL *)url options:(NSDictionary<UIApplicationOpenURLOptionsKey,id> *)options
 {
+    __weak typeof(self) weakSelf = self;
+    [self validatePasscodeIfNeededWithCompletion:^{
+        if (!_libraryViewController) {
+            [weakSelf setupLibrary];
+        }
+        [weakSelf openURL:url options:options];
+    }];
+    return YES;
+}
+
+- (BOOL)openURL:(NSURL *)url options:(NSDictionary<UIApplicationOpenURLOptionsKey,id> *)options {
     //Handles Dropbox Authorization flow.
     DBOAuthResult *authResult = [DBClientsManager handleRedirectURL:url];
     if (authResult != nil) {
@@ -346,12 +366,12 @@ didFailToContinueUserActivityWithType:(NSString *)userActivityType
                 [self playWithURL:movieURL completion:^(BOOL success) {
                     NSURL *callback = success ? successCallback : errorCallback;
                     if (@available(iOS 10, *)) {
-                         [[UIApplication sharedApplication] openURL:callback options:@{} completionHandler:nil];
+                        [[UIApplication sharedApplication] openURL:callback options:@{} completionHandler:nil];
                     } else {
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wdeprecated-declarations"
                         /* UIApplication's replacement calls require iOS 10 or later, which we can't enforce as of yet */
-                       [[UIApplication sharedApplication] openURL:callback];
+                        [[UIApplication sharedApplication] openURL:callback];
 #pragma clang diagnostic pop
                     }
                 }];
@@ -369,7 +389,7 @@ didFailToContinueUserActivityWithType:(NSString *)userActivityType
 
                     /* Safari & al mangle vlc://http:// so fix this */
                     if (location != NSNotFound && [parsedString characterAtIndex:location - 1] != 0x3a) { // :
-                            parsedString = [NSString stringWithFormat:@"%@://%@", [parsedString substringToIndex:location], [parsedString substringFromIndex:location+2]];
+                        parsedString = [NSString stringWithFormat:@"%@://%@", [parsedString substringToIndex:location], [parsedString substringFromIndex:location+2]];
                     } else {
                         parsedString = [receivedUrl substringFromIndex:6];
                         if (![parsedString hasPrefix:@"http://"] && ![parsedString hasPrefix:@"https://"] && ![parsedString hasPrefix:@"ftp://"]) {
