@@ -2,7 +2,7 @@
  * VLCMovieViewController.m
  * VLC for iOS
  *****************************************************************************
- * Copyright (c) 2013-2017 VideoLAN. All rights reserved.
+ * Copyright (c) 2013-2018 VideoLAN. All rights reserved.
  * $Id$
  *
  * Authors: Felix Paul Kühne <fkuehne # videolan.org>
@@ -24,7 +24,6 @@
 #import "VLCEqualizerView.h"
 #import "VLCMultiSelectionMenuView.h"
 #import "VLCPlaybackController.h"
-#import "UIDevice+VLC.h"
 #import "VLCTimeNavigationTitleView.h"
 #import "VLCPlayerDisplayController.h"
 #import "VLCAppDelegate.h"
@@ -54,7 +53,7 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
   VLCPanTypeProjection
 };
 
-@interface VLCMovieViewController () <UIGestureRecognizerDelegate, VLCMultiSelectionViewDelegate, VLCEqualizerViewUIDelegate, VLCPlaybackControllerDelegate, VLCDeviceMotionDelegate, VLCRendererDiscovererManagerDelegate>
+@interface VLCMovieViewController () <UIGestureRecognizerDelegate, VLCMultiSelectionViewDelegate, VLCEqualizerViewUIDelegate, VLCPlaybackControllerDelegate, VLCDeviceMotionDelegate, VLCRendererDiscovererManagerDelegate, PlaybackSpeedViewDelegate>
 {
     BOOL _controlsHidden;
     BOOL _videoFiltersHidden;
@@ -105,7 +104,6 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
 
     UIView *_sleepTimerContainer;
     UIDatePicker *_sleepTimeDatePicker;
-    NSTimer *_sleepCountDownTimer;
 
     NSInteger _mediaDuration;
     NSInteger _numberOfTapSeek;
@@ -121,7 +119,10 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
 }
 @property (nonatomic, strong) VLCMovieViewControlPanelView *controllerPanel;
 @property (nonatomic, strong) UIPopoverController *masterPopoverController;
+@property (nonatomic, strong) VLCTimeNavigationTitleView *timeNavigationTitleView;
 @property (nonatomic, strong) IBOutlet PlayingExternallyView *playingExternalView;
+@property (nonatomic, strong) IBOutlet PlaybackSpeedView *playbackSpeedView;
+@property (nonatomic, strong) IBOutlet NSLayoutConstraint *scrubViewTopConstraint;
 
 @end
 
@@ -142,7 +143,6 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
 - (void)viewDidLoad
 {
     [super viewDidLoad];
-    CGRect rect;
 
     _vpc = [VLCPlaybackController sharedInstance];
 
@@ -161,16 +161,8 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
     _saturationSlider.accessibilityLabel = _saturationLabel.text;
     _gammaLabel.text = NSLocalizedString(@"VFILTER_GAMMA", nil);
     _gammaSlider.accessibilityLabel = _gammaLabel.text;
-    _playbackSpeedLabel.text = NSLocalizedString(@"PLAYBACK_SPEED", nil);
-    _playbackSpeedSlider.accessibilityLabel = _playbackSpeedLabel.text;
-    _audioDelayLabel.text = NSLocalizedString(@"AUDIO_DELAY", nil);
-    _audioDelaySlider.accessibilityLabel = _audioDelayLabel.text;
-    _spuDelayLabel.text = NSLocalizedString(@"SPU_DELAY", nil);
-    _spuDelaySlider.accessibilityLabel = _spuDelayLabel.text;
 
     _resetVideoFilterButton.accessibilityLabel = NSLocalizedString(@"VIDEO_FILTER_RESET_BUTTON", nil);
-    _sleepTimerButton.accessibilityLabel = NSLocalizedString(@"BUTTON_SLEEP_TIMER", nil);
-    [_sleepTimerButton setTitle:NSLocalizedString(@"BUTTON_SLEEP_TIMER", nil) forState:UIControlStateNormal];
 
     _multiSelectionView = [[VLCMultiSelectionMenuView alloc] init];
     _multiSelectionView.delegate = self;
@@ -182,6 +174,7 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
 
     self.playbackSpeedView.hidden = YES;
     _playbackSpeedViewHidden = YES;
+    _playbackSpeedView.delegate = self;
     NSNotificationCenter *center = [NSNotificationCenter defaultCenter];
     [center addObserver:self selector:@selector(handleExternalScreenDidConnect:)
                    name:UIScreenDidConnectNotification object:nil];
@@ -206,7 +199,7 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
     _previousJumpState = VLCMovieJumpStateDefault;
     _numberOfTapSeek = 0;
 
-    rect = self.resetVideoFilterButton.frame;
+    CGRect rect = self.resetVideoFilterButton.frame;
     rect.origin.y = rect.origin.y + 5.;
     self.resetVideoFilterButton.frame = rect;
 
@@ -346,6 +339,7 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
                              [NSLayoutConstraint constraintWithItem:_trackSelectorContainer attribute:NSLayoutAttributeWidth relatedBy:NSLayoutRelationLessThanOrEqual toItem:nil attribute:NSLayoutAttributeWidth multiplier:1 constant:420.0],
                              widthConstraint,
                              [NSLayoutConstraint constraintWithItem:_trackSelectorContainer attribute:NSLayoutAttributeHeight relatedBy:NSLayoutRelationLessThanOrEqual toItem:self.view attribute:NSLayoutAttributeHeight multiplier:2.0/3.0 constant:0],
+                             [_videoFilterView.bottomAnchor constraintEqualToAnchor:_controllerPanel.topAnchor]
                              ];
     [NSLayoutConstraint activateConstraints:constraints];
 
@@ -359,8 +353,10 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
     [_doneButton setTitleColor:[UIColor grayColor] forState:UIControlStateDisabled];
     _doneButton.translatesAutoresizingMaskIntoConstraints = NO;
 
-    self.timeNavigationTitleView = [[[NSBundle mainBundle] loadNibNamed:@"VLCTimeNavigationTitleView" owner:self options:nil] objectAtIndex:0];
-    self.timeNavigationTitleView.translatesAutoresizingMaskIntoConstraints = NO;
+    if (!self.timeNavigationTitleView) {
+        self.timeNavigationTitleView = [[[NSBundle mainBundle] loadNibNamed:@"VLCTimeNavigationTitleView" owner:self options:nil] objectAtIndex:0];
+        self.timeNavigationTitleView.translatesAutoresizingMaskIntoConstraints = NO;
+    }
 
     if (_vpc.renderer != nil) {
         [_rendererButton setSelected:YES];
@@ -390,6 +386,7 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
                                               [NSLayoutConstraint constraintWithItem:_navigationBarStackView attribute:NSLayoutAttributeBottom relatedBy:NSLayoutRelationEqual toItem:self.navigationController.navigationBar attribute:NSLayoutAttributeBottom multiplier:1 constant:0],
                                               [NSLayoutConstraint constraintWithItem:_timeNavigationTitleView attribute:NSLayoutAttributeHeight relatedBy:NSLayoutRelationEqual toItem:_navigationBarStackView attribute:NSLayoutAttributeHeight multiplier:1 constant:0]
                                               ]];
+    self.scrubViewTopConstraint.constant = CGRectGetMaxY(self.navigationController.navigationBar.frame);
 }
 
 - (void)resetVideoFiltersSliders
@@ -479,6 +476,8 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
         multiSelectionFrame.origin.y = controllerPanelFrame.origin.y - multiSelectionFrame.size.height;
     }
     _multiSelectionView.frame = multiSelectionFrame;
+
+    self.scrubViewTopConstraint.constant = CGRectGetMaxY(self.navigationController.navigationBar.frame);
 }
 
 - (void)viewWillDisappear:(BOOL)animated
@@ -763,6 +762,9 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
     if (_controlsHidden && !_videoFiltersHidden)
         _videoFiltersHidden = YES;
 
+    if (_controlsHidden && !_playbackSpeedViewHidden)
+        _playbackSpeedViewHidden = YES;
+
     if (_isTapSeeking)
         _numberOfTapSeek = 0;
 
@@ -901,7 +903,7 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
 }
 #pragma mark - controls
 
-- (IBAction)closePlayback:(id)sender
+- (void)closePlayback:(id)sender
 {
     _playbackWillClose = YES;
     [_vpc stopPlayback];
@@ -985,22 +987,6 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
     [self.timeNavigationTitleView setNeedsLayout];
 }
 
-- (void)updateSleepTimerButton
-{
-    NSMutableString *title = [NSMutableString stringWithString:NSLocalizedString(@"BUTTON_SLEEP_TIMER", nil)];
-    if (_vpc.sleepTimer != nil && _vpc.sleepTimer.valid) {
-        int remainSeconds = (int)[_vpc.sleepTimer.fireDate timeIntervalSinceNow];
-        int hour = remainSeconds / 3600;
-        int minute = (remainSeconds - hour * 3600) / 60;
-        int second = remainSeconds % 60;
-        [title appendFormat:@"  %02d:%02d:%02d", hour, minute, second];
-    } else {
-        [_sleepCountDownTimer invalidate];
-    }
-
-    [_sleepTimerButton setTitle:title forState:UIControlStateNormal];
-}
-
 #pragma mark - playback controller delegation
 
 - (void)playbackPositionUpdated:(VLCPlaybackController *)controller
@@ -1023,17 +1009,7 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
     self.timeNavigationTitleView.timeDisplayButton.accessibilityLabel = @"";
     [_equalizerView reloadData];
 
-    double playbackRate = controller.playbackRate;
-    self.playbackSpeedSlider.value = log2(playbackRate);
-    self.playbackSpeedIndicator.text = [NSString stringWithFormat:@"%.2fx", playbackRate];
-
-    float audioDelay = controller.audioDelay;
-    self.audioDelaySlider.value = audioDelay;
-    self.audioDelayIndicator.text = [NSString stringWithFormat:@"%d ms", (int) audioDelay];
-
-    float subtitleDelay = controller.subtitleDelay;
-    self.spuDelaySlider.value = subtitleDelay;
-    self.spuDelayIndicator.text = [NSString stringWithFormat:@"%d ms", (int) subtitleDelay];
+    [_playbackSpeedView prepareForMediaPlaybackWithController:controller];
 
     [self _resetIdleTimer];
 }
@@ -1147,7 +1123,7 @@ currentMediaHasTrackToChooseFrom:(BOOL)currentMediaHasTrackToChooseFrom
     [self _resetIdleTimer];
 }
 
-- (IBAction)sleepTimer:(id)sender
+- (void)showSleepTimer
 {
     if (!_playbackSpeedViewHidden)
         self.playbackSpeedView.hidden = _playbackSpeedViewHidden = YES;
@@ -1182,13 +1158,7 @@ currentMediaHasTrackToChooseFrom:(BOOL)currentMediaHasTrackToChooseFrom
 {
     [_vpc scheduleSleepTimerWithInterval:_sleepTimeDatePicker.countDownDuration];
 
-    if (_sleepCountDownTimer == nil || _sleepCountDownTimer.valid == NO) {
-        _sleepCountDownTimer = [NSTimer scheduledTimerWithTimeInterval:1
-                                                                   target:self
-                                                                 selector:@selector(updateSleepTimerButton)
-                                                                 userInfo:nil
-                                                                  repeats:YES];
-    }
+    [_playbackSpeedView setupSleepTimerIfNecessary];
     [self.statusLabel showStatusMessage:NSLocalizedString(@"SLEEP_TIMER_UPDATED", nil)];
     [self setControlsHidden:YES animated:YES];
 }
@@ -1477,7 +1447,7 @@ currentMediaHasTrackToChooseFrom:(BOOL)currentMediaHasTrackToChooseFrom
     if (!_seekGestureEnabled)
         return;
 
-    NSString * hudString = @" ";
+    NSString *hudString = @" ";
 
     int swipeForwardDuration = (_variableJumpDurationEnabled) ? ((int)(_mediaDuration*0.001*0.05)) : FORWARD_SWIPE_DURATION;
     int swipeBackwardDuration = (_variableJumpDurationEnabled) ? ((int)(_mediaDuration*0.001*0.05)) : BACKWARD_SWIPE_DURATION;
@@ -1495,15 +1465,15 @@ currentMediaHasTrackToChooseFrom:(BOOL)currentMediaHasTrackToChooseFrom
             [_vpc jumpForward:(timeRemaining - 5)];
             hudString = [NSString stringWithFormat:@"⇒ %is",(timeRemaining - 5)];
         }
-    }
-    else if (swipeRecognizer.direction == UISwipeGestureRecognizerDirectionLeft) {
+    } else if (swipeRecognizer.direction == UISwipeGestureRecognizerDirectionLeft) {
         [_vpc jumpBackward:swipeBackwardDuration];
         hudString = [NSString stringWithFormat:@"⇐ %is",swipeBackwardDuration];
-    }else if (swipeRecognizer.direction == UISwipeGestureRecognizerDirectionUp) {
+    } else if (swipeRecognizer.direction == UISwipeGestureRecognizerDirectionUp) {
         [self backward:self];
-    }
-    else if (swipeRecognizer.direction == UISwipeGestureRecognizerDirectionDown) {
+        hudString = NSLocalizedString(@"BWD_BUTTON", "");
+    } else if (swipeRecognizer.direction == UISwipeGestureRecognizerDirectionDown) {
         [self forward:self];
+        hudString = NSLocalizedString(@"FWD_BUTTON", "");
     }
 
     if (swipeRecognizer.state == UIGestureRecognizerStateEnded) {
@@ -1552,12 +1522,6 @@ currentMediaHasTrackToChooseFrom:(BOOL)currentMediaHasTrackToChooseFrom
     if (_equalizerView.hidden == NO)
         _equalizerView.hidden = YES;
 
-    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPhone) {
-        if (!_controlsHidden) {
-            self.controllerPanel.hidden = _controlsHidden = YES;
-        }
-    }
-
     self.videoFilterView.hidden = !_videoFiltersHidden;
     _videoFiltersHidden = self.videoFilterView.hidden;
 }
@@ -1594,26 +1558,6 @@ currentMediaHasTrackToChooseFrom:(BOOL)currentMediaHasTrackToChooseFrom
 }
 
 #pragma mark - playback view
-- (IBAction)playbackSliderAction:(UISlider *)sender
-{
-    if (sender == _playbackSpeedSlider) {
-        double speed = exp2(sender.value);
-        _vpc.playbackRate = speed;
-        self.playbackSpeedIndicator.text = [NSString stringWithFormat:@"%.2fx", speed];
-    } else if (sender == _audioDelaySlider) {
-        int delay = ((int) round(sender.value / 50.)) * 50;
-        _vpc.audioDelay = delay;
-        [sender setValue:delay animated:NO];
-        _audioDelayIndicator.text = [NSString stringWithFormat:@"%d ms", delay];
-    } else if (sender == _spuDelaySlider) {
-        int delay = (int) (round(sender.value / 50.)) * 50;
-        _vpc.subtitleDelay = delay;
-        [sender setValue:delay animated:NO];
-        _spuDelayIndicator.text = [NSString stringWithFormat:@"%d ms", delay];
-    }
-
-    [self _resetIdleTimer];
-}
 
 - (IBAction)videoDimensionAction:(id)sender
 {
@@ -1724,4 +1668,17 @@ currentMediaHasTrackToChooseFrom:(BOOL)currentMediaHasTrackToChooseFrom
 {
     [self showOnDisplay:_movieView];
 }
+
+#pragma mark - PlaybackSpeedViewDelegate
+
+- (void)playbackSpeedViewShouldResetIdleTimer:(PlaybackSpeedView *)playbackSpeedView
+{
+    [self _resetIdleTimer];
+}
+
+- (void)playbackSpeedViewSleepTimerHit:(PlaybackSpeedView *)playbackSpeedView
+{
+    [self showSleepTimer];
+}
+
 @end
